--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local AimGuide = {}

local Random = require 'random'
local Utility = require 'utility'
local ScreenManager = require 'screen_manager'
local MouseManager = require 'mouse_manager'

----------------------------------------------------------------------

local COLOUR = {255, 0, 128}
local MAX_ALPHA = 192
local ALPHA_SPEED = 0.1

local NUM_POINTS = 30
local POINT_RADIUS = 3

local player_shoot_y
local points

----------------------------------------------------------------------

local function create_points()
    for i = 1, NUM_POINTS do
        table.insert(points, {
            dist = math.random(),
            age = 2 * math.random()
        })
    end
end

local function load()
    player_shoot_y = 0
    points = {}
    create_points()
end
AimGuide.load = load

local function set_player_shoot_y(shoot_y)
    player_shoot_y = shoot_y
end
AimGuide.set_player_shoot_y = set_player_shoot_y

local function update()
    for _, p in ipairs(points) do
        p.age = p.age + ALPHA_SPEED
        if p.age > 2 then
            p.dist = math.random()
            p.age = p.age - 2
        end
    end
end
AimGuide.update = update

local function draw()
    local screen_width
    local screen_height
    screen_width, screen_height = ScreenManager.dimensions()

    local length = screen_width / 2
    if screen_height < screen_width then
        length = (screen_height - player_shoot_y) / 2
    end

    local mouse_x
    local mouse_y
    mouse_x, mouse_y = MouseManager.get_pos()

    local source_x = screen_width / 2
    local source_y = (screen_height / 2) - player_shoot_y

    local angle = Utility.coords_to_angle(
      mouse_x - source_x, mouse_y - source_y)

    for _, p in ipairs(points) do
        local alpha_mult = 1 - math.abs(p.age - 1)
        local alpha = MAX_ALPHA * alpha_mult

        love.graphics.setColor(
          COLOUR[1], COLOUR[2], COLOUR[3], alpha)

        local range = length * p.dist
        local x = range * math.cos(angle)
        local y = range * math.sin(angle)

        love.graphics.circle(
            'fill', x + source_x, y + source_y,
            POINT_RADIUS * alpha_mult, 12
        )
    end
    love.graphics.setColor(255, 255, 255, 255)
end
AimGuide.draw = draw

----------------------------------------------------------------------

return AimGuide