--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local ColourFlux = {}

local Utility = require 'utility'

----------------------------------------------------------------------

local function new(colours, speed)
    return {
        colours = colours,
        speed = speed,
        index = 0,
        delta = 0
    }
end
ColourFlux.new = new

local function update(flux)
    flux.delta = flux.delta + flux.speed
    if flux.delta > 1 then
        flux.delta = flux.delta - 1
        flux.index = (flux.index + 1) % #flux.colours
    end
end
ColourFlux.update = update

local function reset(flux)
    flux.index = 0
    flux.delta = 0
end
ColourFlux.reset = reset

local function get_colour(flux)
    local colour_a = flux.colours[flux.index + 1]
    local colour_b = flux.colours[
        (flux.index + 1) % #flux.colours + 1
    ]

    return Utility.interpolate_colour(
        colour_a[1], colour_a[2], colour_a[3], colour_a[4],
        colour_b[1], colour_b[2], colour_b[3], colour_b[4],
        flux.delta
    )
end
ColourFlux.get_colour = get_colour

----------------------------------------------------------------------

return ColourFlux