--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local Entrance = {}

local Utility = require 'utility'

----------------------------------------------------------------------

local ENTRANCE_IMG = 'entrance'

local ENTRANCE_WIDTH = 72
local ENTRANCE_PART_HEIGHT = 68

local EXIT_EDGE_HEIGHT = 36
local EXIT_SHIFT_HEIGHT = 11

local HALF_SIZE = ENTRANCE_WIDTH / 2
local GLOW_SIZE = 4

local EXIT_RADIUS = 32

local entrance_img

local entrance_back_quad
local entrance_front_quad

local exit_bottom_quad
local exit_back_quad
local exit_front_quad

----------------------------------------------------------------------

local function init_img()
    entrance_img = love.graphics.newImage(
      Utility.img_path(ENTRANCE_IMG))
end

local function new_entrance(pos)
    if not entrance_img then
        init_img()
    end

    if not entrance_back_quad then
        local img_width = entrance_img:getWidth()
        local img_height = entrance_img:getHeight()

        entrance_back_quad = love.graphics.newQuad(
          0, 0, ENTRANCE_WIDTH, ENTRANCE_PART_HEIGHT,
          img_width, img_height
        )

        entrance_front_quad = love.graphics.newQuad(
          ENTRANCE_WIDTH, 0, ENTRANCE_WIDTH, ENTRANCE_PART_HEIGHT,
          img_width, img_height
        )
    end

    return {pos = pos}
end
Entrance.new_entrance = new_entrance

local function new_exit(phys_world, pos)
    if not entrance_img then
        init_img()
    end

    if not exit_back_quad then
        local img_width = entrance_img:getWidth()
        local img_height = entrance_img:getHeight()

        exit_bottom_quad = love.graphics.newQuad(
          ENTRANCE_WIDTH * 2, 0, ENTRANCE_WIDTH, ENTRANCE_WIDTH,
          img_width, img_height
        )

        exit_back_quad = love.graphics.newQuad(
          0, ENTRANCE_PART_HEIGHT,
          ENTRANCE_WIDTH, EXIT_EDGE_HEIGHT,
          img_width, img_height
        )

        exit_front_quad = love.graphics.newQuad(
          ENTRANCE_WIDTH, ENTRANCE_PART_HEIGHT,
          ENTRANCE_WIDTH, EXIT_EDGE_HEIGHT,
          img_width, img_height
        )
    end

    local body = love.physics.newBody(phys_world, pos.x, pos.y)
    local shape = love.physics.newCircleShape(EXIT_RADIUS)
    local fixture = love.physics.newFixture(body, shape)

    fixture:setSensor(true)

    local result = {
        pos = pos,
        phys_data = {
            body = body,
            shape = shape,
            fixture = fixture
        }
    }

    fixture:setUserData({
        type = 'exit',
        data = result
    })

    return result
end
Entrance.new_exit = new_exit

local function entrance_draw_data(e)
    return {
        {
            img = entrance_img,
            quad = entrance_back_quad,
            x = e.pos.x - HALF_SIZE,
            y = e.pos.y - ENTRANCE_PART_HEIGHT,

            real_y = e.pos.y,
            z = 1
        },
        {
            img = entrance_img,
            quad = entrance_front_quad,
            x = e.pos.x - HALF_SIZE,
            y = e.pos.y - HALF_SIZE + GLOW_SIZE,

            real_y = e.pos.y + HALF_SIZE,
            z = 1
        }
    }
end
Entrance.entrance_draw_data = entrance_draw_data

local function exit_draw_data(e)
    return {
        {
            img = entrance_img,
            quad = exit_bottom_quad,
            x = e.pos.x - HALF_SIZE,
            y = e.pos.y - HALF_SIZE,

            real_y = e.pos.y + HALF_SIZE,
            z = 0
        },
        {
            img = entrance_img,
            quad = exit_back_quad,
            x = e.pos.x - HALF_SIZE,
            y = e.pos.y - HALF_SIZE - EXIT_SHIFT_HEIGHT,

            real_y = e.pos.y - EXIT_SHIFT_HEIGHT,
            z = 1
        },
        {
            img = entrance_img,
            quad = exit_front_quad,
            x = e.pos.x - HALF_SIZE,
            y = e.pos.y - EXIT_SHIFT_HEIGHT,

            real_y = e.pos.y + HALF_SIZE - EXIT_SHIFT_HEIGHT,
            z = 1
        }
    }
end
Entrance.exit_draw_data = exit_draw_data

----------------------------------------------------------------------

return Entrance