--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local Key = {}

local Constants = require 'constants'
local Utility = require 'utility'

----------------------------------------------------------------------

local KEY_IMG = 'keys'

local KEY_WIDTH = 30
local KEY_HEIGHT = 48

local KEY_GLOW_SIZE = 2

local FULL_KEY_WIDTH = KEY_WIDTH + (KEY_GLOW_SIZE * 2)
local FULL_KEY_HEIGHT = KEY_HEIGHT + (KEY_GLOW_SIZE * 2)

Key.FULL_WIDTH = FULL_KEY_WIDTH
Key.FULL_HEIGHT = FULL_KEY_HEIGHT

local KEY_COLUMNS = 2

local key_image
local key_quads

----------------------------------------------------------------------

local function init_image()
    key_image = love.graphics.newImage(Utility.img_path(KEY_IMG))
    key_quads = {}

    local img_width = key_image:getWidth()
    local img_height = key_image:getHeight()

    local x
    local y
    for i = 0, Constants.MAX_DOORS - 1 do
        x = i % KEY_COLUMNS
        y = math.floor(i / KEY_COLUMNS)

        x = x * FULL_KEY_WIDTH
        y = y * FULL_KEY_HEIGHT

        table.insert(key_quads, love.graphics.newQuad(
          x, y,
          FULL_KEY_WIDTH, FULL_KEY_HEIGHT,
          img_width, img_height
        ))
    end
end

local function new(phys_world, index, pos)
    if not key_image then
        init_image()
    end

    local body = love.physics.newBody(phys_world, pos.x, pos.y)
    local shape = love.physics.newRectangleShape(
      KEY_WIDTH, KEY_HEIGHT)
    local fixture = love.physics.newFixture(body, shape)

    fixture:setSensor(true)

    local result = {
        index = index,
        phys_data = {
            body = body,
            shape = shape,
            fixture = fixture
        }
    }

    fixture:setUserData({
        type = 'key',
        data = result
    })

    return result
end
Key.new = new

local function draw_data(key)
    local x = Utility.round(key.phys_data.body:getX())
    local y = Utility.round(key.phys_data.body:getY())

    return {
        img = key_image,
        quad = key_quads[key.index],
        x = x - (FULL_KEY_WIDTH / 2),
        y = y - (FULL_KEY_HEIGHT / 2),

        real_y = y + (FULL_KEY_HEIGHT / 2),
        z = 0
    }
end
Key.draw_data = draw_data

local function draw_sprite(index, x, y)
    love.graphics.drawq(key_image, key_quads[index], x, y)
end
Key.draw_sprite = draw_sprite

----------------------------------------------------------------------

return Key