--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local MouseManager = {}

local Utility = require 'utility'
local ColourFlux = require 'colour_flux'

local ScreenManager = require 'screen_manager'

----------------------------------------------------------------------

local COLOURS = {
    {255,   0,   0, 255}, -- Red
    {255, 128,   0, 255}, -- Orange
    {255, 255,   0, 255}, -- Yellow
    {  0, 255,   0, 255}, -- Green
    {  0, 255, 255, 255}, -- Cyan
    {  0,   0, 255, 255}, -- Blue
    {255,   0, 255, 255}  -- Violet
}
local COLOUR_SPEED = 0.125

local images
local colour_flux

----------------------------------------------------------------------

local function load()
    images = {}
    colour_flux = ColourFlux.new(COLOURS, COLOUR_SPEED)
end
MouseManager.load = load

local function update()
    ColourFlux.update(colour_flux)
end
MouseManager.update = update

local function get_pos()
    local scale = ScreenManager.get_scale()
    return love.mouse.getX() * scale, love.mouse.getY() * scale
end
MouseManager.get_pos = get_pos

local function draw(image_name, dx, dy)
    if image_name then
        if not images[image_name] then
            images[image_name] = love.graphics.newImage(
              Utility.img_path(image_name)
            )
        end

        love.graphics.setColor(ColourFlux.get_colour(colour_flux))

        local x
        local y
        x, y = get_pos()

        love.graphics.draw(images[image_name], x + dx, y + dy)

        love.graphics.setColor(255, 255, 255, 255)
    end
end
MouseManager.draw = draw

----------------------------------------------------------------------

return MouseManager