--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local MysteryBox = {}

local Utility = require 'utility'

----------------------------------------------------------------------

local BOX_IMG = 'mystery_box'

local BOX_SIZE = 32

local BOX_WIDTH = BOX_SIZE
local BOX_HEIGHT = 48

MysteryBox.WIDTH = BOX_WIDTH
MysteryBox.HEIGHT = BOX_HEIGHT

local TRANS_X = BOX_SIZE / 2
local TRANS_Y = BOX_HEIGHT - BOX_SIZE / 2

local box_image
local box_quad

----------------------------------------------------------------------

local function new(phys_world, pos)
    if not box_image then
        box_image = love.graphics.newImage(Utility.img_path(BOX_IMG))
        box_quad = love.graphics.newQuad(
          0, 0, BOX_WIDTH, BOX_HEIGHT,
          box_image:getWidth(), box_image:getHeight()
        )
    end

    local body = love.physics.newBody(phys_world, pos.x, pos.y)
    local shape = love.physics.newRectangleShape(BOX_SIZE, BOX_SIZE)
    local fixture = love.physics.newFixture(body, shape)

    fixture:setSensor(true)

    local result = {
        index = index,
        phys_data = {
            body = body,
            shape = shape,
            fixture = fixture
        }
    }

    fixture:setUserData({
        type = 'mystery_box',
        data = result
    })

    return result
end
MysteryBox.new = new

local function draw_data(box)
    local x = Utility.round(box.phys_data.body:getX())
    local y = Utility.round(box.phys_data.body:getY())

    return {
        img = box_image,
        quad = box_quad,
        x = x - TRANS_X,
        y = y - TRANS_Y,

        real_y = y + (BOX_WIDTH / 2),
        z = 1
    }
end
MysteryBox.draw_data = draw_data

local function draw_sprite(x, y)
    love.graphics.drawq(box_image, box_quad, x, y)
end
MysteryBox.draw_sprite = draw_sprite

----------------------------------------------------------------------

return MysteryBox