--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local Parallax = {}

local Utility = require 'utility'
local ScreenManager = require 'screen_manager'

----------------------------------------------------------------------

local function new(layers)
    local result = {}

    for _, l in ipairs(layers) do
        local img = love.graphics.newImage(Utility.img_path(
          l.image_name))
        local speed = l.speed

        table.insert(result, {
            image = img,
            speed = speed,
            x = 0,
            y = 0
        })
    end

    return result
end
Parallax.new = new

local function move(parallax, dx, dy)
    for _, layer in ipairs(parallax) do
        layer.x = (layer.x + (layer.speed * dx))
          % layer.image:getWidth()
        layer.y = (layer.y + (layer.speed * dy))
          % layer.image:getHeight()
    end
end
Parallax.move = move

local function draw(parallax)
    local screen_width
    local screen_height
    screen_width, screen_height = ScreenManager.dimensions()

    for _, layer in ipairs(parallax) do
        for x = -layer.x, screen_width, layer.image:getWidth() do
            for y = -layer.y, screen_height, layer.image:getHeight()
              do
                love.graphics.draw(layer.image, x, y)
            end
        end
    end
end
Parallax.draw = draw

local function reset(parallax)
    for _, layer in ipairs(parallax) do
        layer.x = 0
        layer.y = 0
    end
end
Parallax.reset = reset

----------------------------------------------------------------------

return Parallax