--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local Particles = {}

local Constants = require 'constants'
local Utility = require 'utility'

----------------------------------------------------------------------

local IMG_PATH = 'particle'

local particle_img

----------------------------------------------------------------------

local function init_img()
    if not particle_img then
        particle_img = love.graphics.newImage(
          Utility.img_path(IMG_PATH)
        )
    end
end

local function create_fountain(x, y, buffer_size, emission_rate,
  radius, min_speed, max_speed)
    init_img()

    local system = love.graphics.newParticleSystem(particle_img,
      buffer_size)

    local lifetime = 1 / (max_speed / radius)

    system:setEmissionRate(emission_rate)
    system:setSpeed(min_speed, max_speed)
    system:setLifetime(-1)
    system:setParticleLife(lifetime)
    system:setSpread(math.pi * 2)
    system:setPosition(x, y)

    system:stop()

    return system
end
Particles.create_fountain = create_fountain

local function create_explosion(x, y, num_particles, radius,
  num_frames)
    init_img()

    local speed = radius / (num_frames * Constants.SECONDS_PER_FRAME)

    local system = love.graphics.newParticleSystem(particle_img,
      num_particles)

    system:setEmissionRate(num_particles * Constants.FPS)
    system:setSpeed(0, speed)
    system:setLifetime(0)
    system:setParticleLife(num_frames * Constants.SECONDS_PER_FRAME)
    system:setSpread(math.pi * 2)
    system:setPosition(x, y)

    system:start()

    return system
end
Particles.create_explosion = create_explosion

local function finished(system)
    return not system:isActive() and system:isEmpty()
end
Particles.finished = finished

local function update(system)
    system:update(Constants.SECONDS_PER_FRAME)
end
Particles.update = update

local function draw(system, pos)
    love.graphics.draw(system, 0, 0)
end
Particles.draw = draw

----------------------------------------------------------------------

return Particles