--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local PersistantGameData = {}

local ActorData = require 'data/actors'

----------------------------------------------------------------------

local player_health
local mystery_boxes
local num_triple_shots
local num_exploding_shots
local invincibility_time

local level_num

----------------------------------------------------------------------

local function load()
    player_health = ActorData['player'].health
    mystery_boxes = 0
    num_triple_shots = 0
    num_exploding_shots = 0
    invincibility_time = 0
    level_num = 0
end
PersistantGameData.load = load

local function new_level()
    level_num = level_num + 1
end
PersistantGameData.new_level = new_level

local function current_level()
    return level_num
end
PersistantGameData.current_level = current_level

local function current_player_health()
    return player_health
end
PersistantGameData.current_player_health = current_player_health

local function current_num_triple_shots()
    return num_triple_shots
end
PersistantGameData.current_num_triple_shots = current_num_triple_shots

local function current_num_exploding_shots()
    return num_exploding_shots
end
PersistantGameData.current_num_exploding_shots
  = current_num_exploding_shots

local function current_invincibility_time()
    return invincibility_time
end
PersistantGameData.current_invincibility_time
  = current_invincibility_time

local function set_player_data(plyr_health, triple_shots,
  exploding_shots, invincibility)
    player_health = plyr_health
    num_triple_shots = triple_shots
    num_exploding_shots = exploding_shots
    invincibility_time = invincibility
end
PersistantGameData.set_player_data = set_player_data

local function get_num_mystery_boxes()
    return mystery_boxes
end
PersistantGameData.get_num_mystery_boxes = get_num_mystery_boxes

local function add_mystery_box()
    mystery_boxes = mystery_boxes + 1
end
PersistantGameData.add_mystery_box = add_mystery_box

local function use_mystery_box()
    local result = false
    if mystery_boxes > 0 then
        mystery_boxes = mystery_boxes - 1
        result = true
    end
    return result
end
PersistantGameData.use_mystery_box = use_mystery_box

local function end_game()
    player_health = ActorData['player'].health
    mystery_boxes = 0
    num_triple_shots = 0
    num_exploding_shots = 0
    invincibility_time = 0
    level_num = 0
end
PersistantGameData.end_game = end_game

----------------------------------------------------------------------

return PersistantGameData