--
-- Copyright 2012 Aaron MacDonald
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--     http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-- implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--

local TileSet = {}

local TileSheets = require 'data/tilesheets'
local Utility = require 'utility'

----------------------------------------------------------------------

local function new(sheet_name)
    local sheet = TileSheets[sheet_name]

    local image = love.graphics.newImage(Utility.img_path(sheet.file))

    local img_width = image:getWidth()
    local img_height = image:getHeight()

    local quads = {}
    for name, q in pairs(sheet.named_tiles) do
        quads[name] = love.graphics.newQuad(
          q[1], q[2],
          q[3], q[4],
          img_width, img_height
        )
    end

    return {
        image = image,
        quads = quads
    }
end
TileSet.new = new

local function draw(tileset, tile_name, x, y)
    local img = tileset.image
    local quad = tileset.quads[tile_name]

    love.graphics.drawq(img, quad, x, y)
end
TileSet.draw = draw

----------------------------------------------------------------------

return TileSet